package com.bizmatics.controller.web;


import com.bizmatics.common.core.bean.ApiResult;
import com.bizmatics.model.Site;
import com.bizmatics.model.SiteList;
import com.bizmatics.model.TemplateData;
import com.bizmatics.service.SiteService;
import com.bizmatics.service.vo.DeviceCountVO;
import com.bizmatics.service.vo.SiteVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

/**
 * 站点
 *
 * @author ya
 * @since 2021-07-07
 */
@RestController
@RequestMapping("/site")
public class SiteController {


    @Autowired
    private SiteService siteService;

    /**
     * 查看不同状态的站点数量
     *
     * @return
     */
    @GetMapping("count")
    public ApiResult<DeviceCountVO> getCount() {
        return ApiResult.success(siteService.selectCount());
    }


    /**
     * 查看站点列表
     *
     * @param name 名称
     * @return
     */
    @GetMapping("list")
    public ApiResult<List<Site>> list(@RequestParam(required = false) String name) {
        return ApiResult.success(siteService.list(name));
    }

    /**
     * @param name 站点名称
     * @return
     */

    @GetMapping("listStatus")
    public ApiResult<List<SiteVO>> siteAndStatus(@RequestParam(required = false) String name) {
        return ApiResult.success(siteService.siteAndStatusList(name));
    }

    /**
     * 查看站点详情
     *
     * @param siteId 站点id
     * @return
     */
    @GetMapping("one")
    public ApiResult<SiteVO> getOne(@RequestParam Long siteId) {
        return ApiResult.success(siteService.getOne(siteId));
    }

    /**
     * 查看站点列表
     *
     * @param name 站点名称
     * @return
     */
    @GetMapping("Sitelist")
    public ApiResult<List<Map<String, Object>>> Sitelist(@RequestParam(required = false) String name) {
        return ApiResult.success(siteService.Sitelist(name));
    }


    /**
     *
     * @param id 站点ID
     * @param siteName 站点名称
     * @param siteAddress 站点地址
     * @param userName
     * @param phone
     * @param stationAreaId
     * @param routeId
     * @param installedCapacity
     * @param rheologicalChange
     * @param longitude
     * @param latitude
     * @param region
     * @param groupingId
     * @param faultStatus
     * @param realTimeLoad
     * @param totalElectricity
     * @param powerFactor
     * @param siteType
     * @param userCompany
     * @param operationTime
     * @param serviceDeadline
     * @param powerSupplyType
     * @param voltageLevel
     * @param demolitionStandardCoal
     * @param demolitionStandardCoal1
     * @param sitePictureOne
     * @param sitePictureTwo
     * @param sitePictureThree
     * @param templateId
     * @return
     */
    @GetMapping("SiteAdd")
    public ApiResult<Void> SiteAdd(@RequestParam(required = false) String id,
                                   @RequestParam(required = false) String siteName,
                                   @RequestParam(required = false) String siteAddress,
                                   @RequestParam(required = false) String userName,
                                   @RequestParam(required = false) String phone,
                                   @RequestParam(required = false) String stationAreaId,
                                   @RequestParam(required = false) String routeId,
                                   @RequestParam(required = false) String installedCapacity,
                                   @RequestParam(required = false) String rheologicalChange,
                                   @RequestParam(required = false) String longitude,
                                   @RequestParam(required = false) String latitude,
                                   @RequestParam(required = false) String region,
                                   @RequestParam(required = false) String groupingId,
                                   @RequestParam(required = false) String faultStatus,
                                   @RequestParam(required = false) String realTimeLoad,
                                   @RequestParam(required = false) String totalElectricity,
                                   @RequestParam(required = false) String powerFactor,
                                   @RequestParam(required = false) String siteType,
                                   @RequestParam(required = false) String userCompany,
                                   @RequestParam(required = false) String operationTime,
                                   @RequestParam(required = false) String serviceDeadline,
                                   @RequestParam(required = false) String powerSupplyType,
                                   @RequestParam(required = false) String voltageLevel,
                                   @RequestParam(required = false) String demolitionStandardCoal,
                                   @RequestParam(required = false) String demolitionStandardCoal1,
                                   @RequestParam(required = false) String sitePictureOne,
                                   @RequestParam(required = false) String sitePictureTwo,
                                   @RequestParam(required = false) String sitePictureThree,
                                   @RequestParam(required = false) String templateId) {
        if (id.equals("")&&id.equals("0")&&id.equals("null")){
            siteService.SiteAdd(siteName, siteAddress, userName, phone,
                    stationAreaId, routeId, installedCapacity, rheologicalChange, longitude,
                    latitude, region, groupingId, faultStatus, realTimeLoad,
                    totalElectricity, powerFactor, siteType, userCompany,
                    operationTime, serviceDeadline, powerSupplyType, voltageLevel,
                    demolitionStandardCoal, demolitionStandardCoal1,sitePictureOne,sitePictureTwo,sitePictureThree,templateId);
        }else {
            siteService.SiteUpdate(id, siteName, siteAddress, userName, phone,
                     stationAreaId, routeId, installedCapacity, rheologicalChange, longitude,
                     latitude, region, groupingId, faultStatus,  realTimeLoad,
                     totalElectricity,  powerFactor,   siteType,  userCompany,
                     operationTime,  serviceDeadline,  powerSupplyType,  voltageLevel,
                     demolitionStandardCoal,  demolitionStandardCoal1, sitePictureOne,
                     sitePictureTwo, sitePictureThree);
        }

        return ApiResult.success();
    }

    @GetMapping("SiteDel")
    public ApiResult<Void> SiteDel(@RequestParam(required = false) String id) {
            siteService.Sitedel(id);
        return ApiResult.success();
    }

    /**
     *
     * @param id
     * @return
     */
    @GetMapping("GetSite")
    public ApiResult<List<TemplateData>> GetSite(@RequestParam String id) {
        List<TemplateData> TemplateDataList = siteService.GetSite(id);
        System.out.println(TemplateDataList);
        return ApiResult.success(siteService.GetSite(id));
    }


    @GetMapping("test1")
    public String test(){
        return "这是中文";
    }
}

